#include <stdio.h>

#include <infiniband/umad.h>
#include <infiniband/mad.h>

#include "mlx_lossy.h"

int supported_device(uint32_t devid) {

	return 1;
#if 0
	switch (devid) {
	case 4099:
	case 48438:
		return 1;
	default:
		return 0;
	}
#endif
}

void usage() {

	printf("\nThis config tool allows you query the lossy VL config for supported Mellanox devices\n");
	printf("Usage: \n\n");
	printf("mlx_lossy [options]\n");
	printf("Options: \n\n");
	printf("\t -c, The HCA to be used to issue the lossy VL MADs.\n"); 
	printf("\t -p, The port used to issue the lossy VL MADs (default is the first active port found).\n");
	printf("\t -d, Enable debug messages\n");
	printf("\t -L, The lid of the target device you want to perform the lossy VL operation on.\n");
	printf("\t -P, The port of the target device you want to perform the lossy VL operation on.\n");
	printf("\t -h, Print this help message\n\n");
}

int main(int argc, char *argv[])
{
	int ret, c;
    uint32_t ignore_buffer_mask = 0;
    uint32_t ignore_credit_mask = 0;

	int mgmt_class = IB_SMI_CLASS;
	uint8_t buf[256];
	lossy_config_t lossy;

	struct ibmad_port *mad_port;
	ib_portid_t ib_portid = {0};


	int ib_port = 0;
	int target_port = 0;
	char *ib_dev = NULL;
	char *lid_str = NULL;
	uint32_t devid;

	
	while ( (c = getopt(argc, argv, "c:hp:P:L:d")) != -1) {
		switch (c) {
		case 'c':
			ib_dev = strdup(optarg);
			break;		
		case 'd':
			ibdebug++;
			madrpc_show_errors(1);
			umad_debug(1);
			break;
		case 'L':
			lid_str = strdup(optarg);
			break;
		case 'p':
			ib_port = atoi(optarg);
			break;
		case 'P':
			target_port = atoi(optarg);
			break;
		case 'h':
		default:
			usage();
			return (EXIT_FAILURE);
		}
	}

	/* 
	 * Need a lid and port to perform the operation on, otherwise fail.
	 */
	if (!lid_str || !target_port) {
		printf("ERROR: You must specify a target lid (-L) and a target port (-P)!\n\n");
		usage();
		return (EXIT_FAILURE);
	}

	/*
	 * Open the device used to send our mads.
	 */
	mad_port = mad_rpc_open_port(ib_dev, ib_port, &mgmt_class, 1);
	if (!mad_port) {
		perror("mad_rpc_open_port");
		return (EXIT_FAILURE);
	}

	ret = ib_resolve_portid_str_via(&ib_portid, lid_str, IB_DEST_LID, 0, mad_port);
	if (ret) {
		printf("ib_resolve_portid_str_via() failed: %d\n", ret);
		return (EXIT_FAILURE);
	}
	
	/*
	 * Get the node attributes of the device (specified by the lid) and
	 * verify it is a Mellanox device that supports lossy VL.
	 */
	memset(&buf, 0, sizeof(buf));
	if (!smp_query_via(&buf, &ib_portid, IB_ATTR_NODE_INFO, 0, 0, mad_port)) {
		printf("subnet_vendor_mad failed\n");
		return (EXIT_FAILURE);
	}

	mad_decode_field(buf, IB_NODE_DEVID_F, &devid);
	if (!supported_device(devid)) {
		printf("device type: %d does not support lossy VL\n", devid);	
		return (EXIT_FAILURE);
	}

	/*
	 * Query the current Lossy VL configuration
	 */
	memset(&buf, 0, sizeof(buf));
	if (!smp_query_via(&buf, &ib_portid, MLX_LOSSY_VL_CFG, (unsigned int)target_port, 0, mad_port)) {
		printf("subnet_vendor_mad failed\n");
		return (EXIT_FAILURE);
	}

	memcpy(&lossy, &buf, sizeof(lossy));

    /*
     * Need to use htohl instead of ntohs because of error from g++ compiler (bug in gcc on
       certain rhel)
     */
    ignore_buffer_mask |= lossy.ignore_buffer_mask;
    ignore_credit_mask |= lossy.ignore_credit_mask;

    printf("Current lossy config for lid: %s, port: %d -- ignore_buffer_mask: 0x%x ignore_credit_mask: 0x%x\n",
            lid_str, target_port, ntohl(ignore_buffer_mask), ntohl(ignore_credit_mask));

	mad_rpc_close_port(mad_port);

	return 0;
}
